@@ -9,7 +9,7 @@ class WarningErrorMixin(object):
 
     @classmethod
     def _get_error_messages(cls):
-        return cls._error_messages.values()
+        return list(cls._error_messages.values())
 
     @classmethod
     def raise_user_error(cls, error, error_args=None,
