@@ -1,10 +1,27 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 import datetime
+from sql.functions import Function
 
+from ... import backend
 from .field import Field
 
 
+class SQLite_Date(Function):
+    __slots__ = ()
+    _function = 'DATE'
+
+
+class SQLite_DateTime(Function):
+    __slots__ = ()
+    _function = 'DATETIME'
+
+
+class SQLite_Time(Function):
+    __slots__ = ()
+    _function = 'TIME'
+
+
 class Date(Field):
     '''
     Define a date field (``date``).
@@ -15,18 +32,25 @@ class Date(Field):
     def sql_format(self, value):
         if value is None:
             return None
-        if isinstance(value, basestring):
-            year, month, day = map(int, value.split("-", 2))
+        if isinstance(value, str):
+            year, month, day = list(map(int, value.split("-", 2)))
             return datetime.date(year, month, day)
 
-        assert(isinstance(value, datetime.date))
-        # Allow datetime with min time for XML-RPC
-        # datetime must be tested separately because datetime is a
-        # subclass of date
-        assert(not isinstance(value, datetime.datetime)
-            or value.time() == datetime.time())
+        if (not isinstance(value, datetime.date)
+                # Allow datetime with min time for XML-RPC
+                # datetime must be tested separately because datetime is a
+                # subclass of date
+                or (isinstance(value, datetime.datetime)
+                    and value.time() != datetime.time())):
+            raise ValueError("invalid type '%s' for %s"
+                % (type(value), type(self)))
         return value
 
+    def sql_cast(self, expression):
+        if backend.name() == 'sqlite':
+            return SQLite_Date(expression)
+        return super(Date, self).sql_cast(expression)
+
 
 class DateTime(Field):
     '''
@@ -53,14 +77,21 @@ class DateTime(Field):
     def sql_format(self, value):
         if not value:
             return None
-        if isinstance(value, basestring):
+        if isinstance(value, str):
             datepart, timepart = value.split(" ")
             year, month, day = map(int, datepart.split("-", 2))
             hours, minutes, seconds = map(int, timepart.split(":"))
             return datetime.datetime(year, month, day, hours, minutes, seconds)
-        assert(isinstance(value, datetime.datetime))
+        if not isinstance(value, datetime.datetime):
+            raise ValueError("invalid type '%s' for %s"
+                % (type(value), type(self)))
         return value.replace(microsecond=0)
 
+    def sql_cast(self, expression):
+        if backend.name() == 'sqlite':
+            return SQLite_DateTime(expression)
+        return super(DateTime, self).sql_cast(expression)
+
 
 class Timestamp(Field):
     '''
@@ -68,11 +99,12 @@ class Timestamp(Field):
     '''
     _type = 'timestamp'
     _sql_type = 'TIMESTAMP'
+    format = '%H:%M:%S.%f'
 
     def sql_format(self, value):
         if value is None:
             return None
-        if isinstance(value, basestring):
+        if isinstance(value, str):
             datepart, timepart = value.split(" ")
             year, month, day = map(int, datepart.split("-", 2))
             timepart_full = timepart.split(".", 1)
@@ -83,7 +115,9 @@ class Timestamp(Field):
                 microseconds = 0
             return datetime.datetime(year, month, day, hours, minutes, seconds,
                     microseconds)
-        assert(isinstance(value, datetime.datetime))
+        if not isinstance(value, datetime.datetime):
+            raise ValueError("invalid type '%s' for %s"
+                % (type(value), type(self)))
         return value
 
 
@@ -97,12 +131,19 @@ class Time(DateTime):
     def sql_format(self, value):
         if value is None:
             return None
-        if isinstance(value, basestring):
+        if isinstance(value, str):
             hours, minutes, seconds = map(int, value.split(":"))
             return datetime.time(hours, minutes, seconds)
-        assert(isinstance(value, datetime.time))
+        if not isinstance(value, datetime.time):
+            raise ValueError("invalid type '%s' for %s"
+                % (type(value), type(self)))
         return value.replace(microsecond=0)
 
+    def sql_cast(self, expression):
+        if backend.name() == 'sqlite':
+            return SQLite_Time(expression)
+        return super(Time, self).sql_cast(expression)
+
 
 class TimeDelta(Field):
     '''
@@ -128,7 +169,9 @@ class TimeDelta(Field):
     def sql_format(self, value):
         if value is None:
             return None
-        assert(isinstance(value, datetime.timedelta))
+        if not isinstance(value, datetime.timedelta):
+            raise ValueError("invalid type '%s' for %s"
+                % (type(value), type(self)))
         return super(TimeDelta, self).sql_format(value)
 
     @classmethod
