@@ -1,11 +1,12 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from sql import Literal, Column
+from sql import Literal, Column, With
 from sql.aggregate import Max
 from sql.conditionals import Coalesce
 from sql.operators import Or
 
-from .field import Field
+from .field import (Field, search_order_validate, context_validate,
+    with_inactive_records)
 from ...pool import Pool
 from ...tools import reduce_ids
 from ...transaction import Transaction
@@ -20,9 +21,10 @@ class Many2One(Field):
 
     def __init__(self, model_name, string='', left=None, right=None,
             ondelete='SET NULL', datetime_field=None, target_search='join',
-            help='', required=False, readonly=False, domain=None, states=None,
-            select=False, on_change=None, on_change_with=None, depends=None,
-            context=None, loading='eager'):
+            search_order=None, search_context=None, help='', required=False,
+            readonly=False, domain=None, states=None, select=False,
+            on_change=None, on_change_with=None, depends=None, context=None,
+            loading='eager'):
         '''
         :param model_name: The name of the target model.
         :param left: The name of the field to store the left value for
@@ -35,6 +37,8 @@ class Many2One(Field):
         :param datetime_field: The name of the field that contains the datetime
             value to read the target record.
         :param target_search: The kind of target search 'subquery' or 'join'
+        :param search_order: The order to use when searching for a record
+        :param search_context: The context to use when searching for a record
         '''
         self.__required = required
         if ondelete not in ('CASCADE', 'RESTRICT', 'SET NULL'):
@@ -55,6 +59,10 @@ class Many2One(Field):
         self.datetime_field = datetime_field
         assert target_search in ['subquery', 'join']
         self.target_search = target_search
+        self.__search_order = None
+        self.search_order = search_order
+        self.__search_context = None
+        self.search_context = search_context or {}
     __init__.__doc__ += Field.__init__.__doc__
 
     def __get_required(self):
@@ -67,6 +75,24 @@ class Many2One(Field):
 
     required = property(__get_required, __set_required)
 
+    @property
+    def search_order(self):
+        return self.__search_order
+
+    @search_order.setter
+    def search_order(self, value):
+        search_order_validate(value)
+        self.__search_order = value
+
+    @property
+    def search_context(self):
+        return self.__search_context
+
+    @search_context.setter
+    def search_context(self, value):
+        context_validate(value)
+        self.__search_context = value
+
     def get_target(self):
         'Return the target Model'
         return Pool().get(self.model_name)
@@ -75,7 +101,7 @@ class Many2One(Field):
         Target = self.get_target()
         if isinstance(value, dict):
             value = Target(**value)
-        elif isinstance(value, (int, long)):
+        elif isinstance(value, int):
             value = Target(value)
         assert isinstance(value, (Target, type(None)))
         super(Many2One, self).__set__(inst, value)
@@ -90,7 +116,7 @@ class Many2One(Field):
         cursor = Transaction().connection.cursor()
         table, _ = tables[None]
         name, operator, ids = domain
-        red_sql = reduce_ids(table.id, ids)
+        red_sql = reduce_ids(table.id, (i for i in ids if i is not None))
         Target = self.get_target()
         left = getattr(Target, self.left).sql_column(table)
         right = getattr(Target, self.right).sql_column(table)
@@ -109,39 +135,32 @@ class Many2One(Field):
 
     def convert_domain_tree(self, domain, tables):
         Target = self.get_target()
+        target = Target.__table__()
         table, _ = tables[None]
         name, operator, ids = domain
-        ids = set(ids)  # Ensure it is a set for concatenation
-
-        def get_child(ids):
-            if not ids:
-                return set()
-            children = Target.search([
-                    (name, 'in', ids),
-                    (name, '!=', None),
-                    ], order=[])
-            child_ids = get_child(set(c.id for c in children))
-            return ids | child_ids
-
-        def get_parent(ids):
-            if not ids:
-                return set()
-            parent_ids = set(getattr(p, name).id
-                for p in Target.browse(ids) if getattr(p, name))
-            return ids | get_parent(parent_ids)
+        red_sql = reduce_ids(target.id, (i for i in ids if i is not None))
 
         if operator.endswith('child_of'):
-            ids = list(get_child(ids))
+            tree = With('id', recursive=True)
+            tree.query = target.select(target.id, where=red_sql)
+            tree.query |= (target
+                .join(tree, condition=Column(target, name) == tree.id)
+                .select(target.id))
         else:
-            ids = list(get_parent(ids))
-        if not ids:
-            expression = Literal(False)
-        else:
-            expression = table.id.in_(ids)
+            tree = With('id', name, recursive=True)
+            tree.query = target.select(
+                target.id, Column(target, name), where=red_sql)
+            tree.query |= (target
+                .join(tree, condition=target.id == Column(tree, name))
+                .select(target.id, Column(target, name)))
+
+        expression = table.id.in_(tree.select(tree.id, with_=[tree]))
+
         if operator.startswith('not'):
             return ~expression
         return expression
 
+    @with_inactive_records
     def convert_domain(self, domain, tables, Model):
         pool = Pool()
         Rule = pool.get('ir.rule')
@@ -165,7 +184,7 @@ class Many2One(Field):
                         return ~expression
                     return expression
 
-                if isinstance(value, basestring):
+                if isinstance(value, str):
                     targets = Target.search([('rec_name', 'ilike', value)],
                         order=[])
                     ids = [t.id for t in targets]
@@ -193,7 +212,7 @@ class Many2One(Field):
                     return ~expression
                 return expression
 
-            if not isinstance(value, basestring):
+            if not isinstance(value, str):
                 return super(Many2One, self).convert_domain(domain, tables,
                     Model)
             else:
