@@ -1,16 +1,20 @@
 # -*- coding: utf-8 -*-
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
+import http.client
 import logging
 import pydoc
+try:
+    from http import HTTPStatus
+except ImportError:
+    from http import client as HTTPStatus
 
-from werkzeug.utils import redirect
 from werkzeug.exceptions import abort
 from sql import Table
 
 from trytond import security
 from trytond import backend
-from trytond.config import config
+from trytond.config import config, get_hostname
 from trytond import __version__
 from trytond.transaction import Transaction
 from trytond.exceptions import (
@@ -18,6 +22,7 @@ from trytond.exceptions import (
     RateLimitException)
 from trytond.tools import is_instance_method
 from trytond.wsgi import app
+from trytond.worker import run_task
 from .wrappers import with_pool
 
 logger = logging.getLogger(__name__)
@@ -48,17 +53,18 @@ def login(request, database_name, user, parameters, language=None):
         Database(database_name).connect()
     except DatabaseOperationalError:
         logger.error('fail to connect to %s', database_name, exc_info=True)
-        abort(404)
+        abort(HTTPStatus.NOT_FOUND)
+    context = {
+        'language': language,
+        '_request': request.context,
+        }
     try:
         session = security.login(
-            database_name, user, parameters, language=language)
-        code = 403
+            database_name, user, parameters, context=context)
+        code = HTTPStatus.UNAUTHORIZED
     except RateLimitException:
         session = None
-        code = 429
-    msg = 'successful login' if session else 'bad login or password'
-    logger.info('%s \'%s\' from %s using %s on database \'%s\'',
-        msg, user, request.remote_addr, request.scheme, database_name)
+        code = HTTPStatus.TOO_MANY_REQUESTS
     if not session:
         abort(code)
     return session
@@ -67,11 +73,9 @@ def login(request, database_name, user, parameters, language=None):
 @app.auth_required
 def logout(request, database_name):
     auth = request.authorization
-    name = security.logout(
-        database_name, auth.get('userid'), auth.get('session'))
-    logger.info('logout \'%s\' from %s using %s on database \'%s\'',
-        name, request.remote_addr, request.scheme, database_name)
-    return True
+    security.logout(
+        database_name, auth.get('userid'), auth.get('session'),
+        context={'_request': request.context})
 
 
 @app.route('/', methods=['POST'])
@@ -83,11 +87,6 @@ def root(request, *args):
     return methods[request.rpc_method](request, *request.rpc_params)
 
 
-@app.route('/', methods=['GET'])
-def home(request):
-    return redirect('/index.html')  # XXX find a better way
-
-
 def db_exist(request, database_name):
     Database = backend.get('Database')
     try:
@@ -97,12 +96,15 @@ def db_exist(request, database_name):
         return False
 
 
-def db_list(*args):
+def db_list(request, *args):
     if not config.getboolean('database', 'list'):
-        raise Exception('AccessDenied')
+        abort(HTTPStatus.FORBIDDEN)
+    context = {'_request': request.context}
+    hostname = get_hostname(request.host)
     with Transaction().start(
-            None, 0, close=True, _nocache=True) as transaction:
-        return transaction.database.list()
+            None, 0, context=context, close=True, _nocache=True
+            ) as transaction:
+        return transaction.database.list(hostname=hostname)
 
 
 @app.auth_required
@@ -140,8 +142,18 @@ def _dispatch(request, pool, *args, **kwargs):
     if method in obj.__rpc__:
         rpc = obj.__rpc__[method]
     else:
-        raise UserError('Calling method %s on %s is not allowed'
-            % (method, obj))
+        abort(HTTPStatus.FORBIDDEN)
+
+    user = request.user_id
+    session = None
+    if request.authorization.type == 'session':
+        session = request.authorization.get('session')
+
+    if rpc.fresh_session and session:
+        context = {'_request': request.context}
+        if not security.check_timeout(
+                pool.database_name, user, session, context=context):
+            abort(http.client.UNAUTHORIZED)
 
     log_message = '%s.%s(*%s, **%s) from %s@%s/%s'
     username = request.authorization.username
@@ -151,14 +163,13 @@ def _dispatch(request, pool, *args, **kwargs):
         obj, method, args, kwargs, username, request.remote_addr, request.path)
     logger.info(log_message, *log_args)
 
-    user = request.user_id
-
     for count in range(config.getint('database', 'retry'), -1, -1):
         with Transaction().start(pool.database_name, user,
                 readonly=rpc.readonly) as transaction:
             try:
                 c_args, c_kwargs, transaction.context, transaction.timestamp \
                     = rpc.convert(obj, *args, **kwargs)
+                transaction.context['_request'] = request.context
                 meth = getattr(obj, method)
                 if (rpc.instantiate is None
                         or not is_instance_method(obj, method)):
@@ -186,12 +197,11 @@ def _dispatch(request, pool, *args, **kwargs):
                 raise
             # Need to commit to unlock SQLite database
             transaction.commit()
-        if request.authorization.type == 'session':
-            try:
-                with Transaction().start(pool.database_name, 0) as transaction:
-                    Session = pool.get('ir.session')
-                    Session.reset(request.authorization.get('session'))
-            except DatabaseOperationalError:
-                logger.debug('Reset session failed', exc_info=True)
+        while transaction.tasks:
+            task_id = transaction.tasks.pop()
+            run_task(pool, task_id)
+        if session:
+            context = {'_request': request.context}
+            security.reset(pool.database_name, session, context=context)
         logger.debug('Result: %s', result)
         return result
