@@ -2,6 +2,7 @@
 # this repository contains the full copyright notices and license terms.
 import datetime
 import os
+import logging
 import subprocess
 import tempfile
 import warnings
@@ -23,13 +24,13 @@ try:
 except ImportError:
     Manifest, MANIFEST = None, None
 from genshi.filters import Translator
-from trytond.config import config
 from trytond.pool import Pool, PoolBase
 from trytond.transaction import Transaction
 from trytond.url import URLMixin
 from trytond.rpc import RPC
 from trytond.exceptions import UserError
-from trytond.tools import get_parent_language
+
+logger = logging.getLogger(__name__)
 
 MIMETYPES = {
     'odt': 'application/vnd.oasis.opendocument.text',
@@ -82,6 +83,7 @@ class TranslateFactory:
         self.cache = {}
 
     def __call__(self, text):
+        from trytond.ir.lang import get_parent_language
         if self.language not in self.cache:
             cache = self.cache[self.language] = {}
             code = self.language
@@ -101,7 +103,14 @@ class TranslateFactory:
                 code = get_parent_language(code)
         return self.cache[self.language].get(text, text)
 
-    def set_language(self, language):
+    def set_language(self, language=None):
+        pool = Pool()
+        Config = pool.get('ir.configuration')
+        Lang = pool.get('ir.lang')
+        if isinstance(language, Lang):
+            language = language.code
+        if not language:
+            language = Config.get_language()
         self.language = language
 
 
@@ -171,7 +180,7 @@ class Report(URLMixin, PoolBase):
             oext = 'zip'
         else:
             oext, content = cls._execute(records, data, action_report)
-        if not isinstance(content, unicode):
+        if not isinstance(content, str):
             content = bytearray(content) if bytes == str else bytes(content)
         return (oext, content, action_report.direct_print, action_report.name)
 
@@ -184,6 +193,9 @@ class Report(URLMixin, PoolBase):
     def _get_records(cls, ids, model, data):
         pool = Pool()
         Model = pool.get(model)
+        Config = pool.get('ir.configuration')
+        Lang = pool.get('ir.lang')
+        context = Transaction().context
 
         class TranslateModel(object):
             _languages = {}
@@ -192,12 +204,17 @@ class Report(URLMixin, PoolBase):
                 self.id = id
                 self._language = Transaction().language
 
-            def set_lang(self, language):
+            def set_lang(self, language=None):
+                if isinstance(language, Lang):
+                    language = language.code
+                if not language:
+                    language = Config.get_language()
                 self._language = language
 
             def __getattr__(self, name):
                 if self._language not in TranslateModel._languages:
-                    with Transaction().set_context(language=self._language):
+                    with Transaction().set_context(
+                            context=context, language=self._language):
                         records = Model.browse(ids)
                     id2record = dict((r.id, r) for r in records)
                     TranslateModel._languages[self._language] = id2record
@@ -212,8 +229,6 @@ class Report(URLMixin, PoolBase):
             def __str__(self):
                 return '%s,%s' % (Model.__name__, self.id)
 
-            def __unicode__(self):
-                return u'%s,%s' % (Model.__name__, self.id)
         return [TranslateModel(id) for id in ids]
 
     @classmethod
@@ -287,51 +302,58 @@ class Report(URLMixin, PoolBase):
         if output_format in MIMETYPES:
             return output_format, data
 
-        fd, path = tempfile.mkstemp(suffix=(os.extsep + input_format),
-            prefix='trytond_')
+        dtemp = tempfile.mkdtemp(prefix='trytond_')
+        path = os.path.join(
+            dtemp, report.report_name + os.extsep + input_format)
         oext = FORMAT2EXT.get(output_format, output_format)
-        with os.fdopen(fd, 'wb+') as fp:
+        with open(path, 'wb+') as fp:
             fp.write(data)
-        cmd = ['unoconv', '--connection=%s' % config.get('report', 'unoconv'),
-            '-f', oext, '--stdout', path]
         try:
-            proc = subprocess.Popen(cmd, stdout=subprocess.PIPE)
-            stdoutdata, stderrdata = proc.communicate()
-            if proc.wait() != 0:
-                raise Exception(stderrdata)
-            return oext, stdoutdata
+            cmd = ['soffice',
+                '--headless', '--nolockcheck', '--nodefault', '--norestore',
+                '--convert-to', oext, '--outdir', dtemp, path]
+            output = os.path.splitext(path)[0] + os.extsep + oext
+            subprocess.check_call(cmd)
+            if os.path.exists(output):
+                with open(output, 'rb') as fp:
+                    return oext, fp.read()
+            else:
+                logger.error(
+                    'fail to convert %s to %s', report.report_name, oext)
+                return input_format, data
         finally:
-            os.remove(path)
+            try:
+                os.remove(path)
+                os.remove(output)
+                os.rmdir(dtemp)
+            except OSError:
+                pass
 
     @classmethod
-    def format_date(cls, value, lang):
+    def format_date(cls, value, lang=None):
         pool = Pool()
         Lang = pool.get('ir.lang')
-        Config = pool.get('ir.configuration')
-
-        if lang:
-            locale_format = lang.date
-            code = lang.code
-        else:
-            locale_format = Lang.default_date()
-            code = Config.get_language()
-        return Lang.strftime(value, code, locale_format)
+        if lang is None:
+            lang = Lang.get()
+        return lang.strftime(value)
 
     @classmethod
     def format_currency(cls, value, lang, currency, symbol=True,
             grouping=True):
         pool = Pool()
         Lang = pool.get('ir.lang')
-
-        return Lang.currency(lang, value, currency, symbol, grouping)
+        if lang is None:
+            lang = Lang.get()
+        return lang.currency(value, currency, symbol, grouping)
 
     @classmethod
     def format_number(cls, value, lang, digits=2, grouping=True,
             monetary=None):
         pool = Pool()
         Lang = pool.get('ir.lang')
-
-        return Lang.format(lang, '%.' + str(digits) + 'f', value,
+        if lang is None:
+            lang = Lang.get()
+        return lang.format('%.' + str(digits) + 'f', value,
             grouping=grouping, monetary=monetary)
 
 
